var Chassis = Class.create();

/**
 * Chassis class for displaying chassis and its containing objects.
 * Required variables:
 *   window.images_dir
 *
 * @author  Leonard Fischer <lfischer@i-doit.org>
 */
Chassis.prototype = {
	/**
	 * Initialize method, gets called when object is created.
	 *
	 * @param   element  string
	 * @param   options  object
	 * @return  Chassis
	 */
	initialize:function (element, options) {
		this.options = options;

		this.options = Object.extend({
			matrix:[], // The chassis matrix.
			devices:[], // The slots assigned devices.
			view:'front', // The view-point of this chassis instance.
			x:0, // The grid width.
			y:0, // The grid height.
			size:2, // The grid size - From 0-5. 0 is the smallest.
			editmode:false, // Defines, if checkboxes shall be displayed.
			collision_callback:Prototype.emptyFunction, // Callback in case of collision-detection.
			options_active:false, // Defines if options shall be displayed.
			option_callback:Prototype.emptyFunction, // Callback for option-click.
			info_active:false, // Defines if options shall be displayed.
			info_callback:Prototype.emptyFunction // Callback for option-click.
		}, options || {});

		// This is handled separately, because of the IE.
		var tbody = new Element('tbody', {id: 'matrix-' + $(element).id});

		// Prepare the table-size, so that the matrix won't get mashed if it reaches the border.
		this.root_element = $(element).update(new Element('table').update(tbody));
		this.element = tbody;

		this.create_chassis();
	},

	/**
	 * Method for adding observers to the corresponding elements.
	 *
	 * @return  Chassis
	 */
	add_observer:function () {
		$(this.element.id).select('input').invoke('on', 'click', this.checkbox_click.bindAsEventListener(this));
		$(this.element.id).select('img.option').invoke('on', 'click', this.options.option_callback);
		$(this.element.id).select('div.title img.info').invoke('on', 'click', this.options.info_callback);

		return this;
	},

	/**
	 * Method for creating the HTML matrix.
	 *
	 * @return  Chassis
	 */
	create_chassis:function () {
		var x,
			y,
			tr,
			td,
			devices,
			device,
			div,
			height,
			width,
			size = this.get_size_in_pixel();

		this.element.update();

		for (y = -1; y < this.options.y; y++) {
			tr = new Element('tr');

			for (x = -1; x < this.options.x; x++) {

				if (y >= 0 && x >= 0) {
					// This might happen on "real-time-resizing".
					if (typeof this.options.matrix[y] === 'undefined') {
						this.options.matrix[y] = [];
					}

					if (typeof this.options.matrix[y][x] === 'undefined') {
						this.options.matrix[y][x] = null;
					}

					if (this.options.matrix[y][x] !== false) {
						td = new Element('td', this.options.matrix[y][x]);

						if (this.options.matrix[y][x] !== null) {
							td.update(new Element('div', {className: 'title', title: this.options.matrix[y][x]['data-slot-title']}).update(this.options.matrix[y][x]['data-slot-title']));
						}

						if (this.options.editmode === true && this.options.matrix[y][x] === null) {
							td.insert(new Element('input', {type: 'checkbox', id: x + '-' + y})).addClassName('edit');
						} else {
							if (devices = this.options.devices[x + '-' + y]) {
								// This is necessary because we can have more than one device per slot.
								for (var i in devices) {

									// We can set some properties here.
									td.addClassName('device')
										.writeAttribute('data-slot-id', devices[0].slotid);

									// Important check, so that we don't iterate through member-functions.
									if (devices.hasOwnProperty(i)) {
										device = devices[i];

										height = Math.floor((device.height * size - 16) / devices.size());
										width = ((device.width * size));

										div = new Element('div', {className: 'device', style: 'height:' + height + 'px; width:' + width + 'px; background:' + device.object_color}).update(device.title);

										if (device.object_color === null) {
											device.object_color = '#FFFFFF';
										}

										if (device.object_id !== null && device.object_id > 0) {
											div.writeAttribute('data-object-id', device.object_id).writeAttribute('data-object-type', device.object_type);
										}

										td.insert(div).setStyle({background: device.object_color});
									}
								}
							}
						}

						if (this.options.options_active === true && td.hasClassName('slot')) {
							td.down('div.title').insert(new Element('img', {src: window.images_dir + 'icons/silk/bullet_arrow_down.png', className: 'option fr'}));
						}

						if (this.options.info_active === true && td.hasClassName('slot')) {
							td.down('div.title').insert(new Element('img', {src: window.images_dir + 'icons/silk/information.png', className: 'info fr'}));
						}

						tr.insert(td);
					}
				} else {
					// Add "edge" TDs so that col- and rowspans won't shrink the view.
					if (y == -1 && x == -1) {
						if (this.element.up('table').down('colgroup')) {
							this.element.up('table').down('colgroup').remove();
						}

						var colgroup = new Element('colgroup'),
							col_width = this.get_size_in_pixel(),
							col_x;

						colgroup.insert(new Element('col', {width: 0, style: 'width:0;'}));
						for (col_x = 0; col_x < this.options.x; col_x ++) {
							colgroup.insert(new Element('col', {width: col_width, style: 'width:' + col_width + 'px;'}))
						}

						this.element.up('table').insert({top: colgroup});
					}

					tr.insert(new Element('td', {className: 'edge-' + ((y == -1) ? ((x == -1) ? 'first' : 'top') : 'side'), style: ((y == -1) ? ((x == -1) ? '' : 'width:' + size + 'px') : 'height:' + size + 'px')}));
				}
			}

			this.element.insert(tr);
		}

		return this.add_observer();
	},

	/**
	 * Method which get called, when clicking on a checkbox inside the matrix.
	 */
	checkbox_click:function () {
		var checkboxes = $$('#' + this.element.id + ' input:checked');

		$(this.element.id).select('td').invoke('removeClassName', 'selected');
		$(this.element.id).select('input').invoke('enable');
		this.options.collision_callback(false);

		switch (checkboxes.length) {
			case 0:
				// Nothing to do here.
				break;
			case 1:
				checkboxes[0].up().addClassName('selected');
				break;
			case 2:
				[checkboxes[0].up(), checkboxes[1].up()].invoke('addClassName', 'selected');

				var check = [checkboxes[0].id.split('-'), checkboxes[1].id.split('-')];

				this.mark_rectangle(parseInt(check[0][0]), parseInt(check[1][0]), parseInt(check[0][1]), parseInt(check[1][1]));

				$$('#' + this.element.id + ' input:not(:checked)').invoke('disable');
				break;
		}
	},

	/**
	 * Method for marking the selected rectangle.
	 *
	 * @param   x1  integer
	 * @param   x2  integer
	 * @param   y1  integer
	 * @param   y2  integer
	 * @return  Chassis
	 */
	mark_rectangle:function (x1, x2, y1, y2) {
		var from_x = ((x1 > x2) ? x2 : x1),
			to_x = ((x1 > x2) ? x1 : x2),
			from_y = ((y1 > y2) ? y2 : y1),
			to_y = ((y1 > y2) ? y1 : y2),
			x,
			y,
			tmp,
			collision = false;

		for (y = from_y; y <= to_y; y++) {
			for (x = from_x; x <= to_x; x++) {
				tmp = $$('#' + this.element.id + ' #' + x + '-' + y)[0];
				// Just to prevent some Javascript errors.
				if (tmp) {
					tmp.up().addClassName('selected');
				} else {
					collision = true;
				}
			}
		}

		// This callback gets called, if the assignment crosses any other slot(s).
		this.options.collision_callback(collision);

		return this;
	},

	/**
	 * Method for reseting the whole table and building it up new.
	 *
	 * @return  Chassis
	 * @todo    At some point "this.element" is beeing lost, so we assign it new. We should check this some time...
	 */
	reset_html:function () {
		this.element = $(this.element.id);

		return this.create_chassis();
	},

	/**
	 * Method for setting the edit mode.
	 *
	 * @param   editmode  boolean
	 * @return  Chassis
	 */
	set_editmode:function (editmode) {
		this.options.editmode = !! editmode;

		return this;
	},

	/**
	 * Method for setting new devices.
	 *
	 * @param   devices  array
	 * @return  Chassis
	 */
	set_devices:function (devices) {
		this.options.devices = devices;

		return this;
	},

	/**
	 * Method for setting new objects.
	 *
	 * @param   matrix  array
	 * @return  Chassis
	 */
	set_matrix:function (matrix) {
		this.options.matrix = matrix;

		return this;
	},

	/**
	 * Set the horizontal size of the matrix.
	 *
	 * @param   x  integer
	 * @return  Chassis
	 */
	set_x:function (x) {
		this.options.x = parseInt(x);

		return this;
	},

	/**
	 * Set the vertical size of the matrix.
	 *
	 * @param   y  integer
	 * @return  Chassis
	 */
	set_y:function (y) {
		this.options.y = parseInt(y);

		return this;
	},

	/**
	 * Sets the size of each matrix-field (from 0-5).
	 *
	 * @param   size  integer
	 * @return  Chassis
	 */
	set_size:function (size) {
		if (size >= 0 && size <= 5) {
			this.options.size = size;
		}

		return this;
	},

	/**
	 * Method for retrieving the current size-option.
	 *
	 * @return  integer
	 */
	get_size:function () {
		return this.options.size;
	},

	/**
	 * Method for retrieving the pixel-size of the currently set size-option.
	 *
	 * @return  integer
	 */
	get_size_in_pixel:function () {
		switch (parseInt(this.options.size)) {
			case 0: return 25; // The fields will not get smaller than this.
			case 1: return 35;
			default:
			case 2: return 45; // Standard size.
			case 3: return 55;
			case 4: return 70;
			case 5: return 85; // Damn huge!
		}
	}
};