/**
 * Static color class for color-calculation.
 *
 * @author  Leonard Fischer <lfischer@i-doit.com>
 */
Color = {
	/**
	 * Calculates the RED, GREEN and BLUE values from a given HEX.
	 *
	 * @param   hex    May be from 6 to 7 characters long (may prepend '#').
	 * @param   alpha  Alpha-transparency from 0 to 100.
	 * @return  mixed  Return value may be an integer or boolean false, if something went wrong.
	 * @author  Leonard Fischer <lfischer@i-doit.org>
	 */
	calc_rgb_from_hex: function calc_rgb_from_hex(hex, alpha) {
		hex = hex.replace(/#/, '');

		// We only allow 6 character HEX values.
		if (hex.length !== 6) {
			return false;
		} // if

		if (Object.isUndefined(alpha)) {
			alpha = 0;
		}

		var red = hex.slice(0, 2),
			green = hex.slice(2, 4),
			blue = hex.slice(4);

		return [parseInt(red, 16), parseInt(green, 16), parseInt(blue, 16), (alpha / 100.0)];
	},

	/**
	 * This method will render a HEX string from given RGB values.
	 *
	 * @param   red
	 * @param   green
	 * @param   blue
	 * @param   prepend
	 * @return  string
	 * @author  Leonard Fischer <lfischer@i-doit.org>
	 */
	calc_hex_from_rgb: function calc_hex_from_rgb(red, green, blue, prepend) {
		if (Object.isUndefined(prepend)) {
			prepend = '#';
		}

		return prepend + ('0' + red.toString(16)).substr(-2) + ('0' + green.toString(16)).substr(-2) + ('0' + blue.toString(16)).substr(-2);
	},

	/**
	 * Returns a string like "rgb(255, 255, 255)" and "rgba(255, 255, 255, 1)".
	 *
	 * @param   hex    May be from 6 to 7 characters long (may append '#').
	 * @param   alpha  Alpha-transparency from 0 to 100.
	 * @return  string
	 * @uses    Color.calc_rgb_from_hex
	 * @author  Leonard Fischer <lfischer@i-doit.org>
	 */
	render_rgb_from_hex: function render_rgb_from_hex(hex, alpha) {
		if (Object.isUndefined(alpha)) {
			alpha = null;
		}

		var rgba = this.calc_rgb_from_hex(hex, alpha);

		if (rgba === false) {
			return '';
		} // if

		if (alpha === null) {
			return 'rgb(' + rgba[0] + ', ' + rgba[1] + ', ' + rgba[2] + ')';
		} // if

		return 'rgba(' + rgba[0] + ', ' + rgba[1] + ', ' + rgba[2] + ', ' + rgba[3] + ')';
	},

	/**
	 * This method returns a color, corresponding to a given percentage:
	 *
	 * @param   percent
	 * @param   prepend
	 * @return  string  The color as HEX string.
	 * @see     http://stackoverflow.com/questions/17525215/calculate-color-values-from-green-to-red
	 */
	retrieve_color_by_percent: function retrieve_color_by_percent(percent, prepend) {
		if (Object.isUndefined(prepend)) {
			prepend = '#';
		}

		// As the function expects a value between 0 and 1, and red = 0° and green = 120° we convert the input to the appropriate hue value.
		var hue = percent * 1.2 / 360;

		// We convert hsl to rgb (saturation 100%, lightness 50%).
		var rgb = this.hsl_to_rgb(hue, 1, .5);

		// We format to css value and return.
		return this.calc_hex_from_rgb(rgb[0], rgb[1], rgb[2], prepend);
	},

	/**
	 * http://stackoverflow.com/questions/2353211/hsl-to-rgb-color-conversion
	 *
	 * Converts an HSL color value to RGB. Conversion formula adapted from http://en.wikipedia.org/wiki/HSL_color_space.
	 * Assumes h, s, and l are contained in the set [0, 1] and returns r, g, and b in the set [0, 255].
	 *
	 * @param   hue         The hue
	 * @param   saturation  The saturation
	 * @param   lightness   The lightness
	 * @return  Array       The RGB representation
	 */
	hsl_to_rgb: function hsl_to_rgb(hue, saturation, lightness) {
		var r, g, b;

		if (saturation == 0) {
			r = g = b = lightness; // achromatic
		} else {
			var q = lightness < 0.5 ? lightness * (1 + saturation) : lightness + saturation - lightness * saturation;
			var p = 2 * lightness - q;
			r = this.hue_to_rgb(p, q, hue + 1 / 3);
			g = this.hue_to_rgb(p, q, hue);
			b = this.hue_to_rgb(p, q, hue - 1 / 3);
		}

		return [Math.floor(r * 255), Math.floor(g * 255), Math.floor(b * 255)];
	},

	/**
	 * Method for retrieving a random color from a given spectrum.
	 *
	 * @param   colorname  Needs to be a color name like "blue", "red", "yellow", ...
	 * @return  Array       The RGB representation
	 */
	get_random_by_spectrum: function get_random_by_spectrum (colorname, as_hex) {
		var hue, rgb;

		if (Object.isUndefined(as_hex)) {
			as_hex = true;
		}

		switch (colorname) {
			case 'red':
				hue = 0;
				break;
			case 'orange':
				hue = 30;
				break;
			case 'yellow':
				hue = 60;
				break;
			case 'yellowgreen':
				hue = 90;
				break;
			case 'green':
				hue = 120;
				break;
			case 'cyan':
				hue = 150;
				break;
			case 'babyblue':
				hue = 180;
				break;
			case 'skyblue':
				hue = 210;
				break;
			case 'blue':
				hue = 240;
				break;
			case 'purple':
				hue = 270;
				break;
			case 'magenta':
				hue = 300;
				break;
			case 'orchid':
				hue = 330;
				break;
		}

		rgb = this.hsl_to_rgb((((hue - 10) + Math.random() * 20) / 360), ((Math.random() * 0.25) + 0.5), ((Math.random() * 0.25) + 0.3));

		if (as_hex) {
			return this.calc_hex_from_rgb.apply(null, rgb);
		} else {
			return rgb;
		}
	},

	/**
	 * Method for calculating the R, G or B value of the given hue.
	 *
	 * @param    p
	 * @param    q
	 * @param    t
	 * @returns  Number
	 */
	hue_to_rgb: function hue_to_rgb(p, q, t) {
		if (t < 0) t += 1;
		if (t > 1) t -= 1;
		if (t < 1 / 6) return p + (q - p) * 6 * t;
		if (t < 1 / 2) return q;
		if (t < 2 / 3) return p + (q - p) * (2 / 3 - t) * 6;
		return p;
	},

	/**
	 * This fancy method calculates the "light" of a color.
	 * Usefull for switching font-colors on light/dark background-colors.
	 *
	 * @param   hex
	 * @return  Number  The "light" value from 0-100.
	 * @author  Leonard Fischer <lfischer@i-doit.org>
	 */
	calc_color_light: function calc_color_light(hex) {
		var rgb = this.calc_rgb_from_hex(hex);

		if (rgb === false) {
			return false;
		} // if

		// Because we get values from 0 to 510 (255+255) we need to divide by 5.1!
		return (Math.max(rgb[0], rgb[1], rgb[2]) + Math.min(rgb[0], rgb[1], rgb[2])) / 5.1;
	},

	/**
	 * Method for getting a random color.
	 *
	 * @param    prepend
	 * @returns  String  A random RGB string (000000 - FFFFFF)
	 * @author   Leonard Fischer <lfischer@i-doit.org>
	 */
	random_rgb: function random_rgb (prepend) {
		if (Object.isUndefined(prepend)) {
			prepend = '#';
		}

		return prepend + ('0' + (Math.random() * 255).toString(16)).substr(-2) +
			('0' + (Math.random() * 255).toString(16)).substr(-2) +
			('0' + (Math.random() * 255).toString(16)).substr(-2);
	}
};