var Rack = Class.create();

/**
 * Rack class for displaying racks and their containing objects.
 * Required variables:
 *   window.images_dir  string  Relative path to the images-directory.
 *   idoit.Translate.get("LC__CMDB__CATS__RACK__REASSIGN_OBJECT");
 *   idoit.Translate.get("LC__CMDB__CATS__RACK__REMOVE_OBJECT");
 *   idoit.Translate.get("LC__UNIVERSAL__TITLE_LINK");
 *
 * @author  Leonard Fischer <lfischer@i-doit.org>
 */
Rack.prototype = {
	/**
	 * Initialize method, gets called when object is created.
	 *
	 * @param   string  element
	 * @param   object  options
	 * @return  Rack
	 */
	initialize:function (element, options) {
		this.element = $(element);
		this.options = options;

		this.options = Object.extend({
			edit_right:true,        // Will be used to add event handlers and stuff.
			objects:[],             // Objects in this rack.
            room_view:false,        // The room view will deactivate some options and display the rack a lot smaller.
			slots:24,               // Available slots in this rack.
			vertical_slots:0,       // Available slots in this rack.
			view:'front',           // The view-point of this rack instance.
			slot_sort:'asc',        // Defines the slot-number sorting.
			object_link:false,      // Define if the object-titles shall link to the object in i-doit.
			object_reassign:false,  // Defines if the objects shall be re-assignable (use a callback!!).
			object_reassign_callback:Prototype.emptyFunction, // Defines the callback method for reassigning an object.
			object_remove:false,    // Defines if the objects shall be removable (use a callback!!).
			object_remove_callback:Prototype.emptyFunction // Defines the callback method for removing an object.
		}, options || {});

        // This will be used for the object option drop-down list.
        this.object_option_callback = this.object_option.bindAsEventListener(this);

		this.create_rack();
	},

	/**
	 * Method for creating the main HTML, rack slots, assigned objects and vertical slots alltogether.
	 *
	 * @return  Rack
	 */
	create_rack:function () {
		var left_slots,
			right_slots,
			spacing = 0,
            table_class = '';

		if (this.options.view == 'front') {
			left_slots = 'left-slots';
			right_slots = 'right-slots';
		} else {
			// We turn this around, because it's the back view.
			left_slots = 'right-slots';
			right_slots = 'left-slots';
		}

        if (this.options.room_view) {
            table_class = 'room-view';
        }

		if (Prototype.Browser.IEVersion == 8) {
			spacing = 1;
		}

		this.element.update(
			new Element('table', {cellPadding:0, cellSpacing:0, width:'100%', className: table_class})
				.update(new Element('tbody')
                    .update(new Element('tr')
                        .update(new Element('td', {className:left_slots}))
                        .insert(new Element('td', {className:'main-slots'}).update(new Element('table', {cellPadding:0, cellSpacing:spacing})))
                        .insert(new Element('td', {className:right_slots}))))
		);

		return this.create_vertical_slots().create_slots();
	},

	/**
	 * Method for only creating the vertical slots.
	 *
	 * @return  Rack
	 */
	create_vertical_slots:function () {
		var fl = 'fl',
			fr = 'fr';
		if (this.options.view == 'rear') {
			fl = 'fr';
			fr = 'fl';
		}

		this.element.down('.left-slots')
			.update(new Element('div', {className:'slot ' + fl, 'data-slot':7}))
			.insert(new Element('div', {className:'slot ' + fl, 'data-slot':1}))
			.insert(new Element('br', {className:'clear'}))
			.insert(new Element('div', {className:'slot ' + fl, 'data-slot':9}))
			.insert(new Element('div', {className:'slot ' + fl, 'data-slot':3}))
			.insert(new Element('br', {className:'clear'}))
			.insert(new Element('div', {className:'slot last ' + fl, 'data-slot':11}))
			.insert(new Element('div', {className:'slot last ' + fl, 'data-slot':5}));

		this.element.down('.right-slots')
			.update(new Element('div', {className:'slot ' + fr, 'data-slot':2}))
			.insert(new Element('div', {className:'slot ' + fr, 'data-slot':8}))
			.insert(new Element('br', {className:'clear'}))
			.insert(new Element('div', {className:'slot ' + fr, 'data-slot':4}))
			.insert(new Element('div', {className:'slot ' + fr, 'data-slot':10}))
			.insert(new Element('br', {className:'clear'}))
			.insert(new Element('div', {className:'slot last ' + fr, 'data-slot':6}))
			.insert(new Element('div', {className:'slot last ' + fr, 'data-slot':12}));

		return this.update_vertical_slots();
	},

	/**
	 * Method for creating the rack slots and the containing objects.
	 *
	 * @return  Rack
	 */
	create_slots:function () {
		var i,
			cnt,
			num,
			el = this.element.down('.main-slots table').update(new Element('tbody')).down(),
			object,
			rowspan = 0,
			display_obj,
			css,
			last;

		for (i = 1; i <= this.options.slots; i++) {
			num = i;
			css = 'row slot-' + parseInt(num);

			// We check for ascending or descanding sorting to label the slots right.
			if (this.options.slot_sort == 'asc') {
				num = ((num < 10) ? '0' + num : num);
			} else {
				num = (this.options.slots - num) + 1;
				css = 'row slot-' + parseInt(num);
				num = ((num < 10) ? '0' + num : num);
			}

			object = null;
			display_obj = false;

			for (cnt in this.options.objects) {
                // Fix for iterating through member methods.
                if (this.options.objects.hasOwnProperty(cnt)) {
                    // Option "4" = Vertical assignment.
                    if (this.options.objects[cnt].option != 4 && this.options.objects[cnt].pos == i) {
                        if (this.options.objects[cnt].insertion == 2 || (this.options.view == 'front' && this.options.objects[cnt].insertion == 1) || (this.options.view == 'rear' && this.options.objects[cnt].insertion == 0)) {
                            object = this.options.objects[cnt];
                        }
                    }
                }
			}

			if (this.options.slots == i) {
				css += ' last';
			}

            if (i%2) {
                css += ' odd';
            } else {
                css += ' even';
            }

			el.insert(
				new Element('tr', {className:css, 'data-slotnum':i})
					.update(new Element('td').update(num + '&nbsp;'))
					.insert(new Element('td', {className:'slot'}).update('<div style="height:' + (this.options.room_view ? 11 : 19) + 'px;"></div>'))
					//.insert(new Element('td', {className:'slot'}).update('&nbsp;'))
					.insert(new Element('td').update('&nbsp;' + num))
			);

			if (rowspan > 0) {
				el.down('tr:last').down('td', 1).remove();
				rowspan--;
			}

			if (object !== null) {
				if (this.options.view == 'front' && object.insertion == 1) {
					display_obj = true;
				} else if (this.options.view == 'rear' && object.insertion == 0) {
					display_obj = true;
				} else if (object.insertion == 2) {
					display_obj = true;
				}

				if (display_obj) {
					rowspan = (object.height - 1);
					try {
						el.down('tr:last')
							.down('td.slot')
							.writeAttribute({rowspan: object.height})
							.setStyle({backgroundColor: object.color, backgroundImage: 'none'})
							.update(this.render_object(object));
					} catch (e) {
						el.down('tr:last').previous('tr').down('td', 1).removeAttribute('rowspan');
						el.down('tr:last td:first').insert({after:new Element('td', {className:'box-red'})
							.update('Positioning error in ')
							.insert(new Element('a', {href: '?objID=' + object.id}).update(object.title))});
					}
				}
			}
		}

		// Add the "last" class to the last TD in the table.
		last = el.down('td.slot:last');

		// For preventing JS errors...
		if (last) {
			last.addClassName('last');
		}

		return this;
	},

	/**
	 * Method for updating the viewable vertical slots (may be called from a onChange-event).
	 *
	 * @param   integer  slot_number
	 * @return  Rack
	 */
	update_vertical_slots:function (slot_number) {
		var vertical_slots = this.options.vertical_slots;

		if (typeof slot_number != 'undefined') {
			vertical_slots = slot_number;
		}

		// We need to append the ID of this instance to not update all found left and right slots on a page.
		$$('#' + this.element.id + ' .left-slots .slot, #' + this.element.id + ' .right-slots .slot').each(function (el) {
			var display_obj = false,
				cnt,
				object,
				slot = el.readAttribute('data-slot');

			if (isNaN(vertical_slots)) {
				el.hide();
			} else {
				for (cnt in this.options.objects) {
                    // Fix for iterating through member methods.
                    if (this.options.objects.hasOwnProperty(cnt)) {
                        if (this.options.objects[cnt].option == 4 && this.options.objects[cnt].pos == slot) {
                            if ((this.options.view == 'front' && this.options.objects[cnt].insertion == 1) || (this.options.view == 'rear' && this.options.objects[cnt].insertion == 0)) {
                                object = this.options.objects[cnt];
                                display_obj = true;
                            }
                        }
                    }
				}

				if (display_obj) {
					// We have to remove the "style" attribute here, because of it's height.
                    if (Prototype.Browser.IE && Prototype.Browser.IEVersion < 9) {
					    el.update(this.render_object(object).setStyle({height: null}).addClassName('rotated')).setStyle({backgroundColor: object.color, backgroundImage: 'none'});
                    } else {
					    el.update(this.render_object(object).setStyle({height: 'auto'}).addClassName('rotated')).setStyle({backgroundColor: object.color, backgroundImage: 'none'});
                    }
				} else {
					el.update(new Element('span', {className: 'rotated'}).update('&nbsp;Slot ' + slot));
				}

				if (parseInt(el.readAttribute('data-slot')) > vertical_slots) {
					el.hide();
				} else {
					el.show();
				}
			}
		}.bind(this));

		return this;
	},

	/**
	 * Method for setting the slot sorting.
	 *
	 * @param   string  sorting
	 * @return  Rack
	 */
	set_slot_sorting:function (sorting) {
		sorting = sorting.toLowerCase();

		if (sorting == 'asc' || sorting == 'desc') {
			this.options.slot_sort = sorting;
		}

		return this;
	},

	/**
	 * Method for setting new objects.
	 *
	 * @param   array  objects
	 * @return  Rack
	 */
	set_objects:function (objects) {
		this.options.objects = objects;

		return this;
	},

	/**
	 * Method for rendering a DIV with object information inside:
	 * Object, Object type and controls for re-arranging and removing.
	 *
	 * @param   object
	 * @return  Element
	 */
	render_object:function (object) {
        var row_height = (this.options.room_view ? 12 : 20),
	        object_title = object.title + ' (' + object.type + ', FF: ' + object.formfactor + ')',
			object_height = ((object.height * row_height) - 1),
			object_element = new Element('div', {'data-object-height': object.height, 'data-object-id': object.id}).setStyle({height: object_height + 'px'});

        if (this.options.room_view) {
            object_title = new Element('a', {href: '?objID=' + object.id}).update(object_title);
        }

		return object_element
			.update(new Element('img', {src: ((object.icon != '')? object.icon: window.images_dir + 'empty.gif'), alt:object.type, className: 'object-icon'}))
			.insert(new Element('span', {id: 'object-' + object.id}).update(object_title))
			.insert(this.render_object_options())
			.insert(new Element('div', {className:'cmdb-marker mouse-help', style:'float:right; background-color:' + object.cmdb_color + ';', title:object.cmdb_status}));
	},

	render_object_options:function () {
        var option = '';

        if (this.options.edit_right &&  !this.options.room_view && (this.options.object_reassign || this.options.object_remove || this.options.link_objects)) {
            option = new Element('img', {
                className: 'fr object-options mouse-pointer',
                src: window.images_dir + 'multiselect.gif',
                style: 'margin-top:2px;'});
            option.on('click', this.object_option_callback);
        }

        return option;
	},

    object_option:function (ev) {
        if ($('contextWrapper')) {
            $('contextWrapper').remove();
        }

        var contextWrapper,
            x = Event.pointerX(ev) - 5,
            y = Event.pointerY(ev) - 5,
            ev_element = ev.findElement(),
            object_title = ev_element.previous('span').innerHTML.stripTags(),
            object_id = ev_element.up('div').readAttribute('data-object-id'),
            object_height = ev_element.up('div').readAttribute('data-object-height'),
            options = new Element('ul', {id: 'eContextMenu'})
                .update(new Element('li', {className: 'head bold p5'}).update(object_title));

        if (this.options.object_link) {
            options.insert(
                new Element('li').update(
                    new Element('a', {href: '?objID=' + object_id})
                        .update(new Element('img', {src: window.images_dir + 'icons/silk/link.png', alt: idoit.Translate.get("LC__UNIVERSAL__TITLE_LINK")}))
                        .insert('&nbsp;' + idoit.Translate.get("LC__UNIVERSAL__TITLE_LINK"))
                )
            );
        }

        if (this.options.object_reassign) {
            options.insert(
                new Element('li', {'data-object-id': object_id, 'data-object-height': object_height}).update(
                    new Element('a', {href: '#'})
                        .update(new Element('img', {src: window.images_dir + 'icons/silk/arrow_switch.png', alt: idoit.Translate.get("LC__CMDB__CATS__RACK__REASSIGN_OBJECT")}))
                        .insert('&nbsp;' + idoit.Translate.get('LC__CMDB__CATS__RACK__REASSIGN_OBJECT'))
                )
            );

            options.down('li:last').on('click', this.options.object_reassign_callback);
        }

        if (this.options.object_remove) {
            options.insert(
                new Element('li', {'data-object-id': object_id, 'data-object-height': object_height}).update(
                    new Element('a', {href: '#'})
                        .update(new Element('img', {src: window.images_dir + 'icons/silk/cross.png', alt: idoit.Translate.get("LC__CMDB__CATS__RACK__REMOVE_OBJECT")}))
                        .insert('&nbsp;' + idoit.Translate.get('LC__CMDB__CATS__RACK__REMOVE_OBJECT'))
                )
            );

            options.down('li:last').on('click', this.options.object_remove_callback);
        }

        contextWrapper = new Element('div', {id: 'contextWrapper', style: 'top: ' + y + 'px; left: ' + x + 'px'})
            .update(options);
        contextWrapper.on('mouseleave', function () {
            if ($('contextWrapper')) {
                $('contextWrapper').remove();
            }
        });

        $('rackview').insert(contextWrapper);
    }
};